@@ -771,31 +771,40 @@ gb_eval_frx (GBEvalContext *ec,
 	return gb_value_new_frx (v->v.s->str);
 }
 
-static GBValue *
-gb_constant_lookup (GBEvalContext *ec,
-		    GBObjRef      *ref)
+static void
+add_constants (GBEvalContext *ec, GHashTable *consts)
 {
-	static GHashTable       *consts = NULL;
-
 	const GBBuiltinConstInt *i;
-	GBValue                 *val;
 
-	if (ref->parms)
-		return NULL;
-
-	if (consts) {
-		if ((val = g_hash_table_lookup (consts, ref->name)))
-			return gb_value_copy (ec, val);
-		else
-			return NULL;
-	}
-
-	consts = g_hash_table_new (g_str_hash, g_str_equal);
+	/* Integer / long constants */
 	for (i = gb_constant_table; i && i->name; i++)
 		g_hash_table_insert (consts, (char *)i->name,
 				     gb_value_new_long (i->value));
 
-	return gb_constant_lookup (ec, ref);
+	/* Ugly special cases for now */
+	g_hash_table_insert (consts, (char *) "vbCrLf",
+			     gb_value_new_string_chars ("\r\n"));
+}
+
+static GBValue *
+gb_constant_lookup (GBEvalContext *ec,
+		    GBObjRef      *ref)
+{
+	static GHashTable *consts = NULL;
+	GBValue           *val;
+
+	if (ref->parms)
+		return NULL;
+
+	if (!consts) {
+		consts = g_hash_table_new (g_str_hash, g_str_equal);
+		add_constants (ec, consts);
+	}
+
+	if ((val = g_hash_table_lookup (consts, ref->name)))
+		return gb_value_copy (ec, val);
+	else
+		return NULL;
 }
 
 GBValue *
