@@ -200,9 +200,12 @@ parse_basic (YYSTYPE *res, GBLexerStream *ls, char c)
 
 				if (c != '\n' && c != '\r')
 					continue;
-				else
+				else {
+					g_free (str);
 					return GB_KEEP_LOOKING;
+				}
 			}
+			g_free (str);
 			return GB_KEEP_LOOKING;
 		}
 
