@@ -321,17 +321,54 @@ gb_stmt_new_unload (GBParseData *module, const GBExpr *expr)
 	g_return_val_if_fail (module != NULL, NULL);
 
 	stmt = stmt_new (module);
-	stmt->type      = GBS_UNLOAD;
+	stmt->type        = GBS_UNLOAD;
 	stmt->parm.unload = expr;
 
 	return stmt;
 }
 
+const GBStatement *
+gb_stmt_new_redim (GBParseData *module,
+		   const char  *var_name,
+		   GSList      *indices,
+		   gboolean     preserve)
+{
+	GBStatement *stmt;
+
+	g_return_val_if_fail (module != NULL, NULL);
+
+	stmt = stmt_new (module);
+	stmt->type                = GBS_REDIM;
+	stmt->parm.redim.var_name = var_name;
+	stmt->parm.redim.preserve = preserve;
+	stmt->parm.redim.indices  = indices;
+
+	return stmt;
+}
+
+const GBStatement *
+gb_stmt_new_exit (GBParseData   *module,
+		  GBSExitNesting exit)
+{
+	GBStatement *stmt;
+
+	g_return_val_if_fail (module != NULL, NULL);
+
+	stmt = stmt_new (module);
+	stmt->type      = GBS_EXIT;
+	stmt->parm.exit = exit;
+
+	return stmt;
+}
+
 /************** File handling related functions **************************/
 
 const GBStatement *
-gb_stmt_new_open (GBParseData *module, const GBExpr *filename,
-		  OpenMode mode, const GBExpr *handle, const GBExpr *recordlen)
+gb_stmt_new_open (GBParseData  *module,
+		  const GBExpr *filename,
+		  GBSOpenMode   mode,
+		  const GBExpr *handle,
+		  const GBExpr *recordlen)
 {
 	GBStatement *stmt;
 
@@ -644,6 +681,10 @@ gb_stmt_type (const GBStatement *stmt)
 		return "Print";
 	case GBS_SET:
 		return "Set";
+	case GBS_REDIM:
+		return "ReDim";
+	case GBS_EXIT:
+		return "Exit";
 	}
 
 	return "Unknown";
@@ -731,10 +772,18 @@ gb_stmt_print (FILE *sink, const GBStatement *stmt)
 		}
 		break;
 
+	case GBS_REDIM:
+		if (stmt->parm.redim.preserve)
+			fputs ("Preserve ", sink);
+
+		fputs ("Some indices\n", sink);
+		break;
+
 	default:
 		g_warning ("Unknown stmt type %d", stmt->type);
 		return;
 	};
+
 	fputc ('\n', sink);
 }
 
