@@ -27,11 +27,14 @@ typedef enum
     GBS_OPEN = 140, GBS_INPUT = 150, GBS_LINE_INPUT = 160,
     GBS_CLOSE = 170, GBS_ON_ERROR = 180, GBS_GOTO = 190, GBS_LABEL = 200,
     GBS_GET = 210, GBS_PUT = 220, GBS_SEEK = 230, GBS_PRINT = 240,
-    GBS_SET = 250
+    GBS_SET = 250, GBS_REDIM = 280, GBS_EXIT = 290
 } GBStatementType;
 
 typedef enum { GB_OPEN_INPUT, GB_OPEN_OUTPUT, GB_OPEN_APPEND, 
-	       GB_OPEN_BINARY, GB_OPEN_RANDOM } OpenMode; 
+	       GB_OPEN_BINARY, GB_OPEN_RANDOM } GBSOpenMode; 
+
+typedef enum { GBS_EXIT_DO, GBS_EXIT_FOR, GBS_EXIT_FUNCTION,
+	       GBS_EXIT_PROPERTY, GBS_EXIT_SUB } GBSExitNesting;
  
 struct _GBStatement {
 	GBStatementType type;
@@ -89,7 +92,7 @@ struct _GBStatement {
 		/* File handling statements */
 		struct {
 			const GBExpr *filename;
-			OpenMode     mode;
+			GBSOpenMode   mode;
 			const GBExpr *handle;
 			const GBExpr *recordlen;
 		} open;
@@ -124,6 +127,12 @@ struct _GBStatement {
 			const GBExpr *handle;
 			const GBExpr *pos;
 		} seek;
+		struct {
+			const char   *var_name;
+			GSList       *indices;
+			gboolean      preserve;
+		} redim;
+		GBSExitNesting        exit;
 	} parm;
 };
 
@@ -169,15 +178,23 @@ const GBStatement *gb_stmt_new_set        (GBParseData  *module,
 					   gboolean      new,
 					   const GBExpr *objref);
 
-const GBStatement *gb_stmt_new_randomize  (GBParseData       *module,
+const GBStatement *gb_stmt_new_randomize  (GBParseData    *module,
 					   const GBExpr   *expr);
 
-const GBStatement *gb_stmt_new_load       (GBParseData       *module, 
+const GBStatement *gb_stmt_new_load       (GBParseData    *module, 
 					   const GBExpr   *expr);
 
-const GBStatement *gb_stmt_new_unload     (GBParseData       *module,
+const GBStatement *gb_stmt_new_unload     (GBParseData    *module,
 					   const GBExpr   *expr);
 
+const GBStatement *gb_stmt_new_redim      (GBParseData    *module,
+					   const char     *var_name,
+					   GSList         *indices,
+					   gboolean        preserve);
+
+const GBStatement *gb_stmt_new_exit       (GBParseData    *module,
+					   GBSExitNesting  exit);
+
 GSList            *gb_stmt_accumulate     (GSList *list,
 					   const GBStatement *next);
 
@@ -228,7 +245,7 @@ GBSelectCase *gb_select_case_new_csv          (const GBExprList *exprs,
 
 const GBStatement *gb_stmt_new_open           (GBParseData  *module, 
 					       const GBExpr *filename,
-					       OpenMode     mode,
+					       GBSOpenMode   mode,
 					       const GBExpr *handle,
 					       const GBExpr *recordlen);
 
