@@ -208,6 +208,69 @@ gb_value_new_date_gdate (const GDate *date)
 	return v;
 }	
 
+GBValue *
+gb_value_new_date_from_str (const char *str)
+{
+	GBValue *v;
+	char    *d = NULL;
+	char    *t = NULL;
+	int      h = 0, m = 0, s = 0;
+	GDate   *date = g_date_new ();
+
+	/* First we determine if the string is date/time or both */
+	
+	/* 
+	 * FIXME: As of now we don't parse strings such as "Jan 1, 2001" etc 
+	 * We're assuming that dates and times are in the numbers-separated 
+	 * by-slashes-or-colons format 
+	 */
+
+	if (strlen (str) == 8) {
+		if (str [2] == ':') {
+			t = g_strdup (str);
+			d = NULL;
+		} 
+	} 
+
+	if (strlen (str) == 10) {
+		if (str [2] == '/') {
+			d = g_strdup (str);
+			t = NULL;
+		} 
+	}
+	
+	if (!t && !d)
+		return NULL; /* Unrecognized format */
+	  
+	if (t) {
+		h = (t [0] - 48) * 10 + t[1] - 48; /* 0-9 are mapped to ASCII 48-57 */
+		m = (t [3] - 48) * 10 + t[4] - 48;
+		s = (t [6] - 48) * 10 + t[7] - 48;
+		
+		g_free (t);
+	}
+
+	if (d) {
+		g_date_set_parse (date, d);
+		if (!g_date_valid (date))
+			return NULL;
+		
+		g_free (d);
+	}
+	
+	if (d && t)
+		v = gb_value_new_date_time ((const GDate *) date, h, m, s);
+	else if (d)
+		v = gb_value_new_date_gdate ((const GDate *) date);
+	else if (t)
+		v = gb_value_new_time (h, m, s);
+
+	g_date_free (date);
+
+	return v; 
+
+}
+
 GBValue *
 gb_value_new_time (int hr, int min, int sec)
 {
@@ -357,7 +420,7 @@ gb_value_copy (GBEvalContext *ec, const GBValue *v)
 		break;
 		
 	case GB_VALUE_OBJECT:
-		val->v.obj = gb_object_ref (ec, v->v.obj);
+		val->v.obj = gb_object_ref (v->v.obj);
 		break;
 
 	case GB_VALUE_EMPTY:
@@ -765,8 +828,10 @@ gb_value_promote (GBEvalContext *ec,
 	g_return_val_if_fail (v != NULL, NULL);
 	g_return_val_if_fail (ec != NULL, NULL);
 
-	if (v->gtk_type == to || to == gb_gtk_type_from_value (GB_VALUE_VARIANT) ||
-	    gtk_type_is_a (v->gtk_type, to))
+	if (v->gtk_type == to ||
+	    gtk_type_is_a (v->gtk_type, to) ||
+	    to == gb_gtk_type_from_value (GB_VALUE_VARIANT))
+
 		return gb_value_copy (ec, v);
 
 	from = gb_value_from_gtk_type (v->gtk_type);
@@ -1045,18 +1110,10 @@ gb_value_promote (GBEvalContext *ec,
 		switch (from) {
 		case GB_VALUE_STRING:
 		{
-			/* FIXME: Date strings cannot be parsed by a simple
-			 * g_date_set_parse. Write a function to do that so that
-			 * the time part is parsed too */
+			ret = gb_value_new_date_from_str (v->v.s->str);
 
-			GDate *d = g_date_new ();
-			g_date_set_parse (d, v->v.s->str);
-
-			if (!g_date_valid (d))
-				g_warning ("String parse to get a date didn't work.");
-			
-			ret = gb_value_new_date_gdate (d);
-			g_date_free (d);
+			if (!ret) 
+				fprintf (stderr, _("gb_value_promote : String->Date failed on %s"), v->v.s->str);
 			
 			break;
 		}
@@ -1115,11 +1172,14 @@ gb_value_new_default (GBEvalContext *ec, GtkType t)
 
 	g_return_val_if_fail (type_def_hash != NULL, NULL);
 
-	val = g_hash_table_lookup (type_def_hash, GINT_TO_POINTER (t));
-	if (!val)
-		return NULL;
-
-	return gb_value_copy (ec, val);
+	if (gtk_type_is_a (t, gb_fundamental_type ())) {
+		val = g_hash_table_lookup (type_def_hash, GINT_TO_POINTER (t));
+		if (!val)
+			return NULL;
+		
+		return gb_value_copy (ec, val);
+	} else /* Create a new instance and hope */
+		return gb_value_new_object (gtk_type_new (t));
 }
 
 GtkType
@@ -1130,7 +1190,7 @@ gb_gtk_type_from_name (const char *vb_type_name)
 
 	g_return_val_if_fail (vb_type_name != NULL, 0);
 
-	if (!strcmp (vb_type_name, "variant"))
+	if (!g_strcasecmp (vb_type_name, "variant"))
 		return 0;
 
 	name = gb_gtk_type_name (vb_type_name);
@@ -1165,14 +1225,14 @@ add_type_map (const char *name, GBValue *val)
 }
 
 gint
-g_strcase_equal (gconstpointer v, gconstpointer v2)
+gb_strcase_equal (gconstpointer v, gconstpointer v2)
 {
 	return g_strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
 }
 
 /* a char* hash function from ASU */
 guint
-g_strcase_hash (gconstpointer v)
+gb_strcase_hash (gconstpointer v)
 {
 	const unsigned char *s = (const unsigned char *)v;
 	const unsigned char *p;
@@ -1336,14 +1396,14 @@ gb_value_init (void)
 
 	builtin_types [GB_VALUE_OBJECT].created_type =
 		gb_object_get_type ();
-	builtin_types [GB_VALUE_VARIANT].created_type = 0;
+/*	builtin_types [GB_VALUE_VARIANT].created_type = 0;*/
 
 	/* Register various internal types */
 	GBTrue_value  = gb_value_new_boolean (GBTrue);
 	GBFalse_value = gb_value_new_boolean (GBFalse);
 
 	/* Register type name mappings */
-	type_hash     = g_hash_table_new (g_strcase_hash, g_strcase_equal);
+	type_hash     = g_hash_table_new (gb_strcase_hash, gb_strcase_equal);
 	type_def_hash = g_hash_table_new (NULL, NULL); /* hear it type */
 
 	add_type_map ("Null",    gb_value_new_null    ());
