@@ -32,11 +32,11 @@ array_deref (GBRunEvalContext *ec,
 	GBValue   **pos = NULL;
 	GSList     *i, *offset;
 
-	if (g_slist_length (a->indicees) !=
+	if (g_slist_length (a->indices) !=
 	    g_slist_length (ref->parms))
-		return gbrun_exception_firev (ec, _("Too many / few array indicees"));
+		return gbrun_exception_firev (ec, _("Too many / few array indices"));
 
-	i      = a->indicees;
+	i      = a->indices;
 	data   = a->data;
 	offset = ref->parms;
 
@@ -92,7 +92,8 @@ array_deref (GBRunEvalContext *ec,
 
 	if (!data || (!GB_IS_AN_OBJECT (((GBValue *)data)->gtk_type) &&
 		      !GB_IS_A_FUNDAMENTAL (((GBValue *)data)->gtk_type)))
-		return gbrun_exception_firev (ec, _("Serious internal error in array dereference"));
+		return gbrun_exception_firev (ec, _("Unknown type in array dereference '%p' '%s'"),
+					      data, data ? gtk_type_name (((GBValue *)data)->gtk_type): "no data");
 
 	return gb_value_copy (GB_EVAL_CONTEXT (ec), (GBValue *)data);
 }
@@ -180,14 +181,26 @@ range_create (GBRunEvalContext *ec,
 
 	if (r->min > r->max) {
 		GBLong t = r->min;
-		g_warning ("Testme: swapping indicees");
 		r->min = r->max;
 		r->max = t;
+		g_warning ("Testme: swapping indices max %d min %d",
+			   r->max, r->min);
 	}
 
 	return r;
 }
 
+static void
+indices_free (GSList *indices)
+{
+	GSList *l;
+
+	for (l = indices; l; l = l->next)
+		g_free (l->data);
+	
+	g_slist_free (indices);
+}
+
 static gpointer
 alloc_array (GBRunEvalContext *ec,
 	     GSList           *l,
@@ -218,6 +231,30 @@ alloc_array (GBRunEvalContext *ec,
 	return data;
 }
 
+static void
+data_free (GSList *l, gpointer *data)
+{
+	GBRunARange *r;
+	GBLong       size, i;
+
+	if (!l) {
+		if (data)
+			gb_value_destroy ((GBValue *) data);
+		return;
+	}
+
+	r = l->data;
+
+	size = r->max - r->min + 1;
+	if (size < 0)
+		size = -size;
+
+	for (i = 0; i < size; i++)
+		data_free (l->next, data [i]);
+
+	g_free (data);
+}
+
 static GBRunArray *
 array_new_for_type (GBRunEvalContext *ec, const char *type)
 {
@@ -245,18 +282,18 @@ gbrun_array_new (GBRunEvalContext *ec,
 	
 	g_return_val_if_fail (a != NULL, NULL);
 
-	/* Evaluate indicees */
-	a->indicees = NULL;
-	for (l = var->indicees; l; l = l->next) {
+	/* Evaluate indices */
+	a->indices = NULL;
+	for (l = var->indices; l; l = l->next) {
 		GBRunARange *r = range_create (ec, l->data);
 
 		if (!r) /* FIXME: leak */
 			return NULL;
 
-		a->indicees = g_slist_append (a->indicees, r);
+		a->indices = g_slist_append (a->indices, r);
 	}
 
-	a->data = alloc_array (ec, a->indicees, a);
+	a->data = alloc_array (ec, a->indices, a);
 
 	return GB_OBJECT (a);
 }
@@ -281,7 +318,7 @@ gbrun_array_new_vals (GBRunEvalContext *ec,
 	r->min = 0;
 	r->max = g_slist_length (values) - 1;
 
-	a->indicees = g_slist_append (NULL, r);
+	a->indices = g_slist_append (NULL, r);
 
 	v = values->data;
 	a->content_type = v->gtk_type;
@@ -298,6 +335,56 @@ gbrun_array_new_vals (GBRunEvalContext *ec,
 	return GB_OBJECT (a);
 }
 
+gboolean
+gbrun_array_redim (GBRunEvalContext  *ec,
+		   GBRunArray        *a,
+		   const GSList      *new_indices,
+		   gboolean           preserve)
+{
+	GSList *indices;
+	const GSList *l;
+
+	g_return_val_if_fail (GBRUN_IS_ARRAY (a), FALSE);
+
+	/* Evaluate indices */
+	indices = NULL;
+	for (l = new_indices; l; l = l->next) {
+		GBRunARange *r = range_create (ec, l->data);
+		
+		if (!r) /* FIXME: leak */
+			return FALSE;
+		
+		indices = g_slist_append (indices, r);
+	}
+
+	if (preserve) {
+		g_warning ("preserving redim unimplemented");
+		return FALSE;
+	} else {
+		data_free (a->indices, a->data);
+		indices_free (a->indices);
+
+		a->indices = indices;
+		a->data = alloc_array (ec, a->indices, a);
+
+		return TRUE;
+	}
+}
+
+static void
+gbrun_array_destroy (GtkObject *object)
+{
+	GBRunArray *a = GBRUN_ARRAY (object);
+
+	g_warning ("Do array destroy");
+
+	data_free (a->indices, a->data);
+	a->data = NULL;
+
+	indices_free (a->indices);
+	a->indices = NULL;
+}
+
 static void
 gbrun_array_copy (GBEvalContext  *ec,
 		  GBObject       *src,
@@ -309,6 +396,10 @@ gbrun_array_copy (GBEvalContext  *ec,
 static void
 gbrun_array_class_init (GBObjectClass *klass)
 {
+	GtkObjectClass *gtk_class = (GtkObjectClass *) klass;
+
+	gtk_class->destroy = gbrun_array_destroy;
+
 	klass->copy   = gbrun_array_copy;
 	klass->assign = gbrun_array_assign;
 	klass->deref  = gbrun_array_deref;
@@ -337,3 +428,56 @@ gbrun_array_get_type (void)
 
 	return array_type;	
 }
+
+gboolean
+gbrun_array_initialized (GBRunArray *a)
+{
+	g_return_val_if_fail (GBRUN_IS_ARRAY (a), FALSE);
+
+	if (!a->content_type || !a->indices || !a->data)
+		return FALSE;
+	
+	return TRUE;
+}
+
+int
+gbrun_array_dimensions (GBRunArray *a)
+{
+	g_return_val_if_fail (GBRUN_IS_ARRAY (a), 0);
+
+	return g_slist_length (a->indices);
+}
+
+GBValue *
+gbrun_array_lbound (GBRunEvalContext *ec,
+		    GBRunArray       *a,
+		    int               dimension)
+{
+	GSList *l;
+
+	g_return_val_if_fail (GBRUN_IS_ARRAY (a), NULL);
+
+	l = g_slist_nth (a->indices, dimension - 1);
+	if (!l || !l->data)
+		return gbrun_exception_firev (
+			ec, _("dimension %d out of range"), dimension);
+
+	return gb_value_new_long (((GBRunARange *) l->data)->min);
+}
+
+GBValue *
+gbrun_array_ubound (GBRunEvalContext *ec,
+		    GBRunArray       *a,
+		    int               dimension)
+{
+	GSList *l;
+
+	g_return_val_if_fail (GBRUN_IS_ARRAY (a), NULL);
+
+	l = g_slist_nth (a->indices, dimension - 1);
+	if (!l || !l->data)
+		return gbrun_exception_firev (
+			ec, _("dimension %d out of range"), dimension);
+
+	return gb_value_new_long (((GBRunARange *) l->data)->max);
+}
