@@ -23,7 +23,7 @@ struct _GBRunArray {
 
 	GtkType     content_type;
 
-	GSList     *indicees;
+	GSList     *indices;
 	gpointer    data;
 };
 
@@ -31,12 +31,29 @@ struct _GBRunArrayClass {
 	GBObjectClass priv;
 };
 
-GtkType        gbrun_array_get_type (void);
+GtkType        gbrun_array_get_type    (void);
 
-GBObject      *gbrun_array_new      (GBRunEvalContext *ec,
-				     const GBVar      *var);
+GBObject      *gbrun_array_new         (GBRunEvalContext  *ec,
+					const GBVar       *var);
 
-GBObject      *gbrun_array_new_vals (GBRunEvalContext *ec,
-				     GSList           *values);
+GBObject      *gbrun_array_new_vals    (GBRunEvalContext  *ec,
+					GSList            *values);
+
+gboolean       gbrun_array_redim       (GBRunEvalContext  *ec,
+					GBRunArray        *a,
+					const GSList      *new_indices,
+					gboolean           preserve);
+
+gboolean       gbrun_array_initialized (GBRunArray        *a);
+
+int            gbrun_array_dimensions  (GBRunArray        *a);
+
+GBValue       *gbrun_array_lbound      (GBRunEvalContext  *ec,
+					GBRunArray        *a,
+					int                dimension);
+
+GBValue       *gbrun_array_ubound      (GBRunEvalContext  *ec,
+					GBRunArray        *a,
+					int                dimension);
 
 #endif /* GBRUN_ARRAY_H */
