@@ -25,6 +25,9 @@
 #undef VB_METHOD_DEBUG
 #undef VB_INIT_DEBUG
 
+#undef TRACE
+
+
 struct _GBRunObjectPrivate {
 	GBRunStackLevel    *vars;
 };
@@ -32,6 +35,8 @@ struct _GBRunObjectPrivate {
 struct _GBRunObjectPrivateClass {
 	GSList *variables;
 
+	GSList *constants;
+
 	GSList *classes;
 
 	GSList *properties;
@@ -97,6 +102,23 @@ gbrun_object_var_get (GBRunEvalContext *ec,
 	return ret;
 }
 
+static GBValue *
+gbrun_object_const_get (GBRunEvalContext *ec,
+			GBRunObjectClass *klass,
+			const char       *name)
+{
+	GSList *l;
+
+	for (l = klass->priv->constants; l; l = l->next) {
+		GBConst *cons = l->data;
+
+		if (!g_strcasecmp (cons->name, name))
+			return gb_value_copy (GB_EVAL_CONTEXT (ec),
+					      cons->value);
+	}
+	return NULL;
+}
+
 static gboolean
 gbrun_object_set_arg_unimplemented (GBRunEvalContext  *ec,
 				    GBRunObject       *object,
@@ -159,29 +181,33 @@ void
 gbrun_object_add_routine (GBRunObjectClass *klass,
 			  GBRoutine        *routine)
 {
-	GBRunObjMethod *m;
+	GBRunObjMethod	*m;
 
 /*	g_warning ("Adding '%s' to object '%s'\n",
 		   routine->name,
 		   gtk_type_name (GTK_OBJECT_CLASS (klass)->type));*/
 
-	g_return_if_fail (klass != NULL);
+	g_return_if_fail (klass   != NULL);
 	g_return_if_fail (routine != NULL);
 
-	m              = g_new0   (GBRunObjMethod, 1);
+	m                       = g_new0   (GBRunObjMethod, 1);
 
-	m->type        = GBRUN_METHOD_VB;
-	m->name        = g_strdup (routine->name);
-	m->args        = g_slist_copy (routine->args);
-	m->handler.vb  = routine;
+	m->type                 = GBRUN_METHOD_VB;
+	m->name                 = g_strdup (routine->name);
+	m->args_parsed          = TRUE;
+	m->args.parsed.args     = g_slist_copy (routine->args);
+	m->args.parsed.as_type  = routine->as_type ?
+		gb_gtk_type_from_name (routine->as_type) :
+		0;
+	m->handler.vb           = routine;
 	/* FIXME: need to split parse_args code */
-	m->max_args    = g_slist_length (routine->args);
-	m->min_args    = m->max_args;
+	m->args.parsed.max_args = g_slist_length (routine->args);
+	m->args.parsed.min_args = m->args.parsed.max_args;
 	/* No pure VB can have security implications */
-	m->mask        = 0;
-	m->is_sub      = !routine->is_function;
+	m->args.parsed.mask     = 0;
+	m->is_sub               = !routine->is_function;
 	
-	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
+	klass->priv->methods    = g_slist_prepend (klass->priv->methods, m);
 }
 
 static void
@@ -190,6 +216,15 @@ add_routine (gpointer key, gpointer value, gpointer user_data)
 	gbrun_object_add_routine (user_data, value);
 }
 
+
+/**
+ * gbrun_object_add_routines
+ * @ec:
+ * @klass:   the class to add it to.
+ * @routine: the routine structures
+ * 
+ * Add VB routines to an existing class.
+ **/
 void
 gbrun_object_add_routines (GBRunEvalContext *ec,
 			   GBRunObjectClass *klass,
@@ -205,20 +240,16 @@ gbrun_object_add_routines (GBRunEvalContext *ec,
 }
 
 static void
-gbrun_object_add_variable (GBRunObjectClass *klass,
-			   GBVar            *var)
+add_variables (gpointer key, gpointer value, gpointer user_data)
 {
+	GBRunObjectClass *klass = user_data;
+	GBVar            *var   = value;
+
 	/* FIXME: we should dup var really */
 	klass->priv->variables = g_slist_prepend (
 		klass->priv->variables, var);
 }
 
-static void
-add_variables (gpointer key, gpointer value, gpointer user_data)
-{
-	gbrun_object_add_variable (user_data, value);
-}
-
 void
 gbrun_object_add_variables (GBRunEvalContext *ec,
 			    GBRunObjectClass *klass,
@@ -239,6 +270,34 @@ gbrun_object_add_variables (GBRunEvalContext *ec,
 			      add_variables, klass);
 }
 
+static void
+add_constant (gpointer key, gpointer value, gpointer user_data)
+{
+	GBRunObjectClass *klass = user_data;
+	GBConst          *cons  = value;
+
+	/* FIXME: we should dup cons really */
+	klass->priv->constants = g_slist_prepend (
+		klass->priv->constants, cons);
+}
+
+void
+gbrun_object_add_constants (GBRunEvalContext *ec,
+			    GBRunObjectClass *klass,
+			    GHashTable       *constants)
+{
+	g_return_if_fail (ec != NULL);
+	g_return_if_fail (klass != NULL);
+
+	if (!constants)
+		return;
+
+/*	g_warning ("Adding %d constants",
+	g_hash_table_size (constants));*/
+
+	g_hash_table_foreach (constants, add_constant, klass);
+}
+
 /*
  * FIXME: ugly, sluggish, tedious.
  */
@@ -355,26 +414,108 @@ parse_security (const char *txt)
 	return 0;
 }
 
+/**
+ * parse_args
+ *   @m: GBRunObjMethod containing unparsed arguments.
+ *
+ * Parses the previously saved unparsed arguments.
+ **/
 static void
-parse_arg_desc (GBRunObjMethod *m, const char *arg_desc,
-		gboolean vararg)
+parse_args (GBRunObjMethod *m)
+{
+	char	 *unparsed_args;
+
+	gchar	**txt;
+
+	int	  i;
+	int	  max;
+
+	gboolean  vararg;
+
+#ifdef TRACE
+	printf ("parse_args entered.\n");
+#endif
+	if (m->args_parsed)
+		return;
+
+	unparsed_args = g_strdup (m->args.unparsed.args);
+	g_free (m->args.unparsed.args);
+	vararg        = m->args.unparsed.vararg;
+	
+	m->args.parsed.args     = NULL;
+	m->args.parsed.min_args = 0;
+	m->args.parsed.max_args = 0;
+
+	txt = g_strsplit (unparsed_args, ";", -1);
+
+	for (i = 0; txt [i]; i++);
+	max = i;
+
+	if (max < 1)
+		g_error ("parse_args: Must have at least 1 sections in '%s'", unparsed_args);
+
+	m->args.parsed.mask = parse_security (txt [max - 1]);
+
+	for (i = max - (m->is_sub?1:2) - 1; i >= 0; i--) {
+		GBArgDesc *arg;
+
+		if (txt [i] [0] == '.')
+			break;
+
+		arg = parse_arg (txt [i]);
+		
+		m->args.parsed.args = g_slist_prepend (m->args.parsed.args, arg);
+		if (!arg->optional)
+			m->args.parsed.min_args++;
+		m->args.parsed.max_args++;
+	}
+
+	if (m->is_sub)
+		m->args.parsed.as_type = 0;
+	else
+		m->args.parsed.as_type = gb_gtk_type_from_name (txt [max - 2]);
+
+	if (vararg)
+		m->args.parsed.max_args = G_MAXINT;	
+
+	m->args_parsed = TRUE;
+
+	g_strfreev (txt);
+	g_free (unparsed_args);
+}
+
+
+/**
+ * save_arg_desc
+ *   @a
+ *   @arg_desc
+ *   vararg
+ *
+ * Internal function to save arguments
+ **/
+static void
+save_arg_desc (GBRunObjMethod *m, const char *arg_desc, gboolean vararg)
 {
 	gchar  **txt;
-	int      i, max;
 
+	int	 i;
+	int	 max;
+
+#ifdef TRACE
+	printf ("save_arg_desc entered.\n");
+#endif
 	g_return_if_fail (arg_desc != NULL);
 
-	m->args = NULL;
-	m->min_args = 0;
-	m->max_args = 0;
+	m->args_parsed = FALSE;
 
-	txt = g_strsplit (arg_desc, ";", -1);
+	txt = g_strsplit (arg_desc, ";", 2);
 
 	for (i = 0; txt [i]; i++);
 	max = i;
 
 	if (max < 3)
-		g_error ("Must have at least 3 sections in '%s'", arg_desc);
+		g_error ("save_arg_desc: Must have at least 3 sections in '%s'", arg_desc);
+
 
 	g_strchomp (txt [0]);
 	if (!g_strcasecmp (txt [0], "sub")) {
@@ -384,29 +525,14 @@ parse_arg_desc (GBRunObjMethod *m, const char *arg_desc,
 	} else
 		g_error ("Incorrect sub/func specifier '%s'", txt [0]);
 
-	m->name = g_strdup (txt [1]);
-	m->mask = parse_security (txt [max - 1]);
-
-	for (i = max - (m->is_sub?1:2) - 1; i >= 2; i--) {
-		GBArgDesc *arg;
-
-		if (txt [i] [0] == '.')
-			break;
-
-		arg = parse_arg (txt [i]);
-		
-		m->args = g_slist_prepend (m->args, arg);
-		if (!arg->optional)
-			m->min_args++;
-		m->max_args++;
-	}
-
-	if (vararg)
-		m->max_args = G_MAXINT;	
-
+	m->name                 = g_strdup (txt [1]);
+	m->args.unparsed.args   = g_strdup (txt [2]);
+	m->args.unparsed.vararg = vararg;
+	
 	g_strfreev (txt);
 }
 
+
 /**
  * gbrun_object_add_method_var:
  * @klass:  the class to add it to.
@@ -423,13 +549,13 @@ gbrun_object_add_method_var (GBRunObjectClass *klass,
 {
 	GBRunObjMethod *m;
 
-	g_return_if_fail (desc != NULL);
+	g_return_if_fail (desc  != NULL);
 	g_return_if_fail (klass != NULL);
 
 	m              = g_new0   (GBRunObjMethod, 1);
 	m->type        = GBRUN_METHOD_VAR;
 	m->handler.var = method;
-	parse_arg_desc (m, desc, TRUE);
+	save_arg_desc (m, desc, TRUE);
 	
 	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
 }
@@ -455,7 +581,7 @@ gbrun_object_add_method_arg (GBRunObjectClass *klass,
 	m              = g_new0   (GBRunObjMethod, 1);
 	m->type        = GBRUN_METHOD_ARG;
 	m->handler.arg = method;
-	parse_arg_desc (m, desc, FALSE);
+	save_arg_desc (m, desc, FALSE);
 	
 	klass->priv->methods = g_slist_prepend (klass->priv->methods, m);
 }
@@ -608,7 +734,7 @@ gbrun_object_has_property (GBRunObjectClass *klass,
 }
 
 static GBValue *
-gbrun_method_invoke_arg (GBRunEvalContext *ec, GBRunObject      *obj,
+gbrun_method_invoke_arg (GBRunEvalContext *ec, GBRunObject    *obj,
 			 GBRunObjMethod   *m,  const GBObjRef *func)
 {
 	GBValue         *val;
@@ -624,26 +750,29 @@ gbrun_method_invoke_arg (GBRunEvalContext *ec, GBRunObject      *obj,
 
 	len = g_slist_length (func->parms);
 
-	if (len < m->min_args)
+	if (! m->args_parsed)
+		parse_args (m);
+
+	if (len < m->args.parsed.min_args)
 		return gbrun_exception_firev (
 			ec, _("Too few args to %s %d expected %d given"),
-			func->name, m->min_args, len);
+			func->name, m->args.parsed.min_args, len);
 
-	if (len > m->max_args)
+	if (len > m->args.parsed.max_args)
 		return gbrun_exception_firev (
 			ec, _("Too many args to %s %d expected %d given"),
-			func->name, m->max_args, len);
+			func->name, m->args.parsed.max_args, len);
 	
-	vals = g_new (GBValue *, m->max_args + 1);
+	vals = g_new (GBValue *, m->args.parsed.max_args + 1);
 
 	/*
 	 * FIXME: see instr for something that will cock this up; we need to work
 	 * out how many optional args to use, and use them as we go along.
 	 */
-	num_opt = len - m->min_args;
+	num_opt = len - m->args.parsed.min_args;
 	exception = FALSE;
 	lp = func->parms;
-	for (lm = m->args; lm; i++, lm = lm->next) {
+	for (lm = m->args.parsed.args; lm; i++, lm = lm->next) {
 		GBArgDesc *arg = lm->data;
 		GBExpr    *expr;
 
@@ -682,7 +811,7 @@ gbrun_method_invoke_arg (GBRunEvalContext *ec, GBRunObject      *obj,
 }
 
 static GBValue *
-gbrun_method_invoke_var (GBRunEvalContext *ec, GBRunObject      *obj,
+gbrun_method_invoke_var (GBRunEvalContext *ec, GBRunObject    *obj,
 			 GBRunObjMethod   *m,  const GBObjRef *func)
 {
 	int              len;
@@ -692,10 +821,13 @@ gbrun_method_invoke_var (GBRunEvalContext *ec, GBRunObject      *obj,
 
 	len = g_slist_length (func->parms);
 
-	if (len < m->min_args)
+	if (! m->args_parsed)
+		parse_args (m);
+
+	if (len < m->args.parsed.min_args)
 		return gbrun_exception_firev (
 			ec, _("Too few args to %s %d expected %d given"),
-			func->name, m->min_args, len);
+			func->name, m->args.parsed.min_args, len);
 
 	return m->handler.var (ec, obj, func->parms); 
 }
@@ -728,12 +860,15 @@ gbrun_method_invoke_vb (GBRunEvalContext *ec, GBRunObject      *obj,
 
 	len = g_slist_length (func->parms);
 
-	if (len < m->min_args)
+	if (! m->args_parsed)
+		parse_args (m);
+
+	if (len < m->args.parsed.min_args)
 		return gbrun_exception_firev (ec, _("Too few args to %s %d expected %d given"),
-					    func->name, m->min_args, len);
-	if (len > m->max_args)
+					    func->name, m->args.parsed.min_args, len);
+	if (len > m->args.parsed.max_args)
 		return gbrun_exception_firev (ec, _("Too many args to %s %d expected %d given"),
-					    func->name, m->max_args, len);
+					    func->name, m->args.parsed.max_args, len);
 
 	/* Set 'Me' up */
 	old_me = gbrun_eval_context_me_get (ec);
@@ -761,7 +896,7 @@ gbrun_method_invoke_vb (GBRunEvalContext *ec, GBRunObject      *obj,
 	 */
 	gbrun_stack_call (ec, func->name);
 
-	def = m->args;
+	def = m->args.parsed.args;
 	l   = parms;
 	while (l && def) {
 		GBArgDesc *arg = def->data;
@@ -787,6 +922,15 @@ gbrun_method_invoke_vb (GBRunEvalContext *ec, GBRunObject      *obj,
 	fprintf (stderr, ")\n");
 #endif
 
+	if (!m->is_sub) {
+		/* Add a blank return var. to the stack */
+		GBValue *v;
+
+		v = gb_value_new_default (GB_EVAL_CONTEXT (ec),
+					  m->args.parsed.as_type);
+		gbrun_stack_add (ec, m->name, v, GBRUN_STACK_LOCAL);
+	}
+
 	/* Setup the functions variables */
 	if (m->handler.vb->variables)
 		g_hash_table_foreach (m->handler.vb->variables,
@@ -979,12 +1123,20 @@ gbrun_object_deref (GBEvalContext  *gb_ec,
 
 	klass = GBRUN_OBJECT_GET_CLASS (obj);
 
-	/* 2. See if we have a method */
+	/* 2. See if we have a constant */
+	if (!ref->parms &&
+	    (val = gbrun_object_const_get (ec, klass, ref->name)))
+		return val;
+
+	/* 3. See if we have a method */
 	if ((m = gbrun_object_get_method (klass, ref->name))) {
 		GBObjRef parmref;
 		GSList  *parms;
 
-		if (ec->flags & m->mask) {
+		if (! m->args_parsed)
+			parse_args (m);
+			
+		if (ec->flags & m->args.parsed.mask) {
 			if (!try_deref)
 				gbrun_exception_firev (
 					ec, _("Security block on function '%s'"),
@@ -998,7 +1150,7 @@ gbrun_object_deref (GBEvalContext  *gb_ec,
 		parms = ref->parms;
 
 		/* This copes with no-arg functions that return Arrays etc. */
-		if (m->max_args == 0)
+		if (m->args.parsed.max_args == 0)
 			parmref.parms = NULL;
 		else
 			parms = NULL;
@@ -1039,7 +1191,7 @@ gbrun_object_deref (GBEvalContext  *gb_ec,
 			}
 		}
 
- 	} else { /* 3. Fall back to an argument */
+ 	} else { /* 4. Fall back to an argument */
 		GBRunObjectProperty *prop;
 
 		if (!obj)
