@@ -6,7 +6,6 @@
  *
  * Copyright 2000, Helix Code, Inc
  */
-
 #include <gbrun/gbrun.h>
 #include <gb/libgb.h>
 #include <gb/gb-main.h>
@@ -28,26 +27,37 @@ struct _GBRunProjectPriv {
 	GBRunStreamProvider *provider;
 	gpointer             user_data;
 	GBProject           *gb_proj;
-	GSList              *objects;
-
-	GHashTable          *classes;
+	GHashTable          *objects;
+	GSList              *modules;
 };
 
+static gboolean
+obj_release (gpointer key, gpointer value, gpointer user_data)
+{
+	g_free (key);
+	gb_object_unref (value);
+
+	return TRUE;
+}
+
 static void
 project_destroy (GtkObject *obj)
 {
 	GBRunProject     *proj = GBRUN_PROJECT (obj);
 	GBRunProjectPriv *priv = proj->priv;
+	GSList           *l;
 
 	gb_project_destroy (priv->gb_proj);
 	priv->gb_proj = NULL;
 
-	while (priv->objects) {
-		GBObject *obj = priv->objects->data;
+	g_hash_table_foreach_remove (priv->objects, obj_release, NULL);
+	g_hash_table_destroy (priv->objects);
+	priv->objects = NULL;
 
-		gb_object_unref (obj);
-		priv->objects = g_slist_remove (priv->objects, obj);
-	}
+	for (l = priv->modules; l; l = l->next)
+		gb_object_unref (l->data);
+	g_slist_free (priv->modules);
+	priv->modules = NULL;
 
 	g_free (priv);
 }
@@ -74,40 +84,43 @@ gbrun_project_assign (GBEvalContext  *ec,
 }
 
 static GBValue *
-gbrun_project_deref (GBEvalContext  *gb_ec,
+gbrun_project_deref (GBEvalContext  *ec,
 		     GBObject       *obj,
 		     const GBObjRef *ref,
 		     gboolean        try_deref)
 {
 	GSList *l;
+	GBObject *o;
+	GBValue *ret = NULL;
 	GBRunProject *proj = GBRUN_PROJECT (obj);
 
 #ifdef PROJECT_DEBUG
-	g_warning ("In proj deref '%d'", g_slist_length (proj->priv->objects));
+	g_warning ("In proj deref '%d'",
+		   g_hash_table_size (proj->priv->objects));
 #endif
 
-	for (l = proj->priv->objects; l; l = l->next) {
-		char *str = gtk_object_get_data (l->data, "Hack");
+	/* FIXME: consider public / private here */
+	if ((o = g_hash_table_lookup (proj->priv->objects, ref->name))) {
+#ifdef PROJECT_DEBUG
+		g_warning ("Found object '%s' on project ", ref->name);
+#endif
+		return gb_value_new_object (o);
+	}
+
+	for (l = proj->priv->modules; l; l = l->next) {
+		if ((ret = gb_object_deref (ec, GB_OBJECT (l->data),
+					    ref, TRUE))
+		    || gb_eval_exception (ec)) {
+#ifdef PROJECT_DEBUG
+			g_warning ("Found method '%s' on a module", ref->name);
+#endif
+			return ret;
+		}
+	}
 
 #ifdef PROJECT_DEBUG
-		g_warning ("Comparing '%s' with registered object '%s'",
-			   ref->name, str);
+	g_warning ("Found no method '%s' on module", ref->name);
 #endif
-
-		if (!g_strcasecmp (ref->name, str))
-			return gb_value_new_object (l->data);
-	}
-
-		/* FIXME: consider public / private here */
-	for (l = proj->priv->objects; l; l = l->next) {
-		GBValue *ret;
-		
-		if ((ret = gb_object_deref (gb_ec, GB_OBJECT (l->data),
-					    ref, TRUE)) ||
-		    gb_eval_exception (gb_ec))
-			return ret;
-	}
-
 	return NULL;
 }
 
@@ -128,6 +141,8 @@ static void
 gbrun_project_init (GBRunProject *proj)
 {
 	proj->priv = g_new0 (GBRunProjectPriv, 1);
+	proj->priv->objects = g_hash_table_new (
+		gb_strcase_hash, gb_strcase_equal);
 }
 
 GtkType
@@ -195,8 +210,9 @@ add_data_to_class (GBRunEvalContext  *ec,
 	 * for now, a Private type is a pipe-dream, all are public.
 	 */
 	gbrun_register_types (ec, klass, pd->types);
-	gbrun_object_add_routines (ec, klass, pd->routines);
+	gbrun_object_add_routines  (ec, klass, pd->routines);
 	gbrun_object_add_variables (ec, klass, pd->variables);
+	gbrun_object_add_constants (ec, klass, pd->constants);
 
 	return klass;
 }
@@ -223,6 +239,26 @@ parsed_load (GBRunEvalContext    *ec,
 	return pd;
 }
 
+void
+gbrun_project_register_module (GBRunProject *project,
+			       GBObject     *object)
+{
+	project->priv->modules = g_slist_prepend (
+		project->priv->modules, gb_object_ref (object));
+}
+
+void
+gbrun_project_deregister_module (GBRunProject *project,
+				 GBObject     *object)
+{
+	if (g_slist_find (project->priv->modules, object)) {
+		project->priv->modules = g_slist_remove (
+			project->priv->modules, object);
+		gb_object_unref (object);
+	} else
+		g_warning ("Can't find module object to remove");
+}
+
 GBRunProject *
 gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		   GBRunStreamProvider *provider,
@@ -244,7 +280,6 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 	priv->provider  = provider;
 	priv->user_data = user_data;
 	priv->gb_proj   = p;
-	priv->classes   = g_hash_table_new (g_str_hash, g_str_equal);
 
 	gbrun_eval_context_proj_push (ec, proj);
 
@@ -253,7 +288,9 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		const GBParseData *pd;
 		GBRunObject       *obj;
 
+#ifdef PROJECT_DEBUG
 		fprintf (stderr, "Loading module '%s'\n", pp->filename);
+#endif
 
 		gbrun_eval_context_set_module (ec, pp->filename);
 
@@ -263,11 +300,15 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		
 		obj = gbrun_object_new (
 			add_data_to_class (
-				ec, get_class (pp->filename, pd), pd));
+				ec, get_class (pp->name, pd), pd));
+
+		/* FIXME: this should be attached to the object somehow */
+		gb_options_copy (&ec->options, &pd->options);
 
 /*		gb_parse_data_destroy (pd);*/
 
-		gbrun_project_register_object (proj, obj, pp->filename);
+		gbrun_project_register_module (proj, GB_OBJECT (obj));
+		gb_object_unref (GB_OBJECT (obj));
 	}
 
 	for (l = p->classes; l; l = l->next) {
@@ -275,7 +316,9 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		const GBParseData *pd;
 		GBRunObject       *obj;
 
+#ifdef PROJECT_DEBUG
 		fprintf (stderr, "Loading class '%s'\n", pp->filename);
+#endif
 
 		gbrun_eval_context_set_module (ec, pp->filename);
 
@@ -284,11 +327,14 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 
 		obj = gbrun_object_new (
 			add_data_to_class (
-				ec, get_class (pp->filename, pd), pd));
+				ec, get_class (pp->name, pd), pd));
 /*		gb_parse_data_destroy (pd);*/
 		
-		g_hash_table_insert (priv->classes, (gpointer) pp->name, (gpointer) obj); 
+		/* FIXME: this should be attached to the object somehow */
+		gb_options_copy (&ec->options, &pd->options);
 
+		gbrun_project_register_object (proj, pp->name, obj);
+		gb_object_unref (GB_OBJECT (obj));
 	}
 		
 	for (l = p->forms; l; l = l->next) {
@@ -296,7 +342,9 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 		const GBParseData *pd;
 		GBRunObject       *obj;
 
+#ifdef PROJECT_DEBUG
 		fprintf (stderr, "Loading form '%s'\n", (char *)l->data);
+#endif
 
 		gbrun_eval_context_set_module (ec, filename);
 
@@ -308,11 +356,14 @@ gbrun_project_new (GBRunEvalContext *ec, GBProject *p,
 				ec, get_class ("Unused", pd), pd));
 
 #ifdef HAVE_FORM_CODE	
-		gbrun_project_register_object (proj, obj, pd->form->name);
+		gbrun_project_register_object (proj, pd->form->name, obj);
+		gb_object_unref (GB_OBJECT (obj));
 
 		gbrun_form_init (ec, GBRUN_FORM (obj), pd);
 		gbrun_form_invoke (ec, GBRUN_FORM (obj), "Form_Load");
 #endif
+		/* FIXME: this should be attached to the object somehow */
+		gb_options_copy (&ec->options, &pd->options);
 
 /*		gb_parse_data_destroy (pd);*/
 	}
@@ -352,7 +403,10 @@ gbrun_project_execute (GBRunEvalContext *ec, GBRunProject *proj)
 		success = !gbrun_eval_context_exception (ec);
 	} else { /* A form */
 /*		g_warning ("Project form : '%s'", startup);*/
+		/* FIXME: think about all this ... */
+		gbrun_eval_context_proj_push (ec, proj);
 		gtk_main ();
+		gbrun_eval_context_proj_pop (ec);
 		success = TRUE;
 	}
 
@@ -384,28 +438,35 @@ gbrun_shutdown (void)
 
 void
 gbrun_project_register_object (GBRunProject *proj,
-			       GBRunObject  *object,
-			       const char   *name)
+			       const char   *name,
+			       GBRunObject  *object)
 {
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (GBRUN_IS_PROJECT (proj));
 
-	if (!g_slist_find (proj->priv->objects, object)) {
-		/* FIXME: disgusting hack; battery dying */
-		gtk_object_set_data (GTK_OBJECT (object), "Hack",
-				     g_strdup (name));
-		proj->priv->objects = g_slist_append (proj->priv->objects, object);
+	if (!g_hash_table_lookup (proj->priv->objects, name)) {
+		g_hash_table_insert (proj->priv->objects, g_strdup (name),
+				     gb_object_ref (GB_OBJECT (object)));
 	} else
 		g_warning ("Registered project object '%s' twice", name);
 }
 
 void
 gbrun_project_deregister_object (GBRunProject *proj,
-				 GBRunObject  *object)
+				 const char   *name)
 {
+	gpointer orig_key;
+	gpointer orig_value;
+
 	g_return_if_fail (GBRUN_IS_PROJECT (proj));
 
-	proj->priv->objects = g_slist_remove (proj->priv->objects, object);
+	if (g_hash_table_lookup_extended (proj->priv->objects, name,
+					  &orig_key, &orig_value)) {
+		g_hash_table_remove (proj->priv->objects, name);
+		g_free (orig_key);
+		gb_object_unref (GB_OBJECT (orig_value));
+	} else
+		g_warning ("Trying to deregister '%s' which is not registered", name);
 }
 
 GBValue *
@@ -447,27 +508,27 @@ gbrun_project_invoke (GBRunEvalContext *ec, GBRunProject *proj,
 	return val;
 }
 
+/*
+ * An ugly hack of a function: do not use.
+ */
 GSList *
 gbrun_project_fn_names (GBRunProject *proj)
 {
 	GSList *ans = NULL;
 	GSList *l;
+	GSList *m, *i;
 
 	g_return_val_if_fail (proj != NULL, NULL);
 	g_return_val_if_fail (proj->priv != NULL, NULL);
 
-	for (l = proj->priv->objects; l; l = l->next) {
-		GBRunObject          *obj = l->data;
-		GSList               *m, *i;
-
-		m = gbrun_object_get_methods (GBRUN_OBJECT_GET_CLASS (obj));
+	for (l = proj->priv->modules; l; l = l->next) {
+		m = gbrun_object_get_methods (GBRUN_OBJECT_GET_CLASS (l->data));
 
 		for (i = m; i; i = i->next) {
 			GBRunObjMethod *method = i->data;
-
+			
 			ans = g_slist_prepend (ans, method->name);
 		}
-		
 		g_slist_free (m);
 	}
 
