@@ -11,6 +11,8 @@
 #include <gbrun/gbrun-eval.h>
 #include <gbrun/gbrun-value.h>
 #include <gbrun/gbrun-stack.h>
+#include <gbrun/gbrun-array.h>
+#include <gbrun/gbrun-collection.h>
 #include <gbrun/gbrun-statement.h>
 #include <gbrun/gbrun-object.h>
 /* for gettimeofday */
@@ -79,7 +81,7 @@ gbrun_frame_stmts_push (GBRunEvalContext *ec, GSList *stmts)
 }
 
 static void
-gbrun_frame_crop_to (GBRunEvalContext *ec, guint depth)
+gbrun_frame_crop_to_depth (GBRunEvalContext *ec, guint depth)
 {
 	GBRunFrame    *rf = gbrun_stack_frame (ec->stack);
 	GBRunSubFrame *sf;
@@ -98,8 +100,24 @@ gbrun_frame_crop_to (GBRunEvalContext *ec, guint depth)
 	}
 }
 
+static void
+gbrun_frame_crop_to_ptr (GBRunEvalContext *ec,
+			 GBRunSubFrame    *frame)
+{
+	GBRunFrame    *rf = gbrun_stack_frame (ec->stack);
+	GBRunSubFrame *sf;
+
+	g_return_if_fail (rf != NULL);
+
+	for (sf = rf->cur; sf != frame; sf = rf->cur) {
+		rf->cur = sf->parent;
+		g_free (sf);
+	}
+}
+
 static GBStatement *
-gbrun_frame_stmt_next (GBRunEvalContext *ec, gboolean *init)
+gbrun_frame_stmt_next (GBRunEvalContext *ec,
+		       gboolean         *init)
 {
 	GBRunFrame    *rf = gbrun_stack_frame (ec->stack);
 	GBRunSubFrame *sf;
@@ -157,7 +175,9 @@ gbrun_frame_stmt_next (GBRunEvalContext *ec, gboolean *init)
 }
 
 static GSList *
-seek_label (GSList *stmts, const char *label, int *depth)
+seek_label (GSList     *stmts,
+	    const char *label,
+	    int        *depth)
 {
 	GSList  *l, *ret = NULL;
 
@@ -176,6 +196,7 @@ seek_label (GSList *stmts, const char *label, int *depth)
 			ret = seek_label (stmt->parm.forloop.body, label, depth);
 			break;
 
+		case GBS_DO:
 		case GBS_WHILE:
 			ret = seek_label (stmt->parm.do_while.body, label, depth);
 			break;
@@ -188,9 +209,11 @@ seek_label (GSList *stmts, const char *label, int *depth)
 			break;
 			
 		case GBS_FOREACH:
-		case GBS_DO:
-			g_warning ("Goto can't cope with flow control stmt %d",
-				   stmt->type);
+			ret = seek_label (stmt->parm.foreach.body, label, depth);
+			break;
+
+/*			g_warning ("Goto can't cope with flow control stmt '%s'",
+			gb_stmt_type (stmt));*/
 		default:
 			break;
 		};
@@ -203,7 +226,8 @@ seek_label (GSList *stmts, const char *label, int *depth)
 }
 
 static gboolean
-handle_stmt_goto (GBRunEvalContext *ec, const char *label)
+gbrun_stmt_goto (GBRunEvalContext *ec,
+		 const char       *label)
 {
 	GBRunFrame    *rf = gbrun_stack_frame (ec->stack);
 	GBRunSubFrame *sf;
@@ -231,7 +255,7 @@ handle_stmt_goto (GBRunEvalContext *ec, const char *label)
 		return FALSE;
 	}
 
-	gbrun_frame_crop_to (ec, depth - 1);
+	gbrun_frame_crop_to_depth (ec, depth - 1);
 	gbrun_frame_stmts_push (ec, label_stmt);
 	
 	return TRUE;
@@ -243,7 +267,8 @@ handle_stmt_goto (GBRunEvalContext *ec, const char *label)
 
 gboolean
 gbrun_stmt_assign (GBRunEvalContext *ec,
-		   const GBExpr *lexpr, const GBExpr *rexpr)
+		   const GBExpr     *lexpr,
+		   const GBExpr     *rexpr)
 {
 	GBValue *rval;
 	gboolean result;
@@ -269,7 +294,8 @@ gbrun_stmt_assign (GBRunEvalContext *ec,
 }
 
 gboolean
-gbrun_stmt_set (GBRunEvalContext *ec, const GBStatement *stmt)
+gbrun_stmt_set (GBRunEvalContext  *ec,
+		const GBStatement *stmt)
 {
 	GBValue *tmp;
 	gboolean result;
@@ -292,9 +318,42 @@ gbrun_stmt_set (GBRunEvalContext *ec, const GBStatement *stmt)
 	return result;
 }
 
+gboolean
+gbrun_stmt_redim (GBRunEvalContext  *ec,
+		  const GBStatement *stmt)
+{
+	GBValue *tmp;
+	GBObjRef r;
+
+	g_return_val_if_fail (stmt->parm.redim.var_name != NULL, FALSE);
+
+	r.method = FALSE;
+	r.name   = stmt->parm.redim.var_name;
+	r.parms  = NULL;
+
+	tmp = gbrun_objref_deref (ec, NULL, &r, TRUE);
+	if (!tmp)
+		return FALSE;
+
+	if (!gtk_type_is_a (tmp->gtk_type, GBRUN_TYPE_ARRAY)) {
+		gbrun_exception_firev (ec, _("Can only redim arrays"));
+		return FALSE;
+	}
+
+	gbrun_array_redim (ec, GBRUN_ARRAY (tmp->v.obj),
+			   stmt->parm.redim.indices,
+			   stmt->parm.redim.preserve);
+
+	gb_value_destroy (tmp);
+	
+	return TRUE;
+}
+
 static gboolean
-handle_stmt_case (GBRunEvalContext *ec, GBValue *val, GBSelectCase *c,
-		  gboolean *err)
+gbrun_stmt_case (GBRunEvalContext *ec,
+		 GBValue          *val,
+		 GBSelectCase     *c,
+		 gboolean         *err)
 {
 	gboolean ret = FALSE;
 
@@ -428,7 +487,9 @@ handle_stmt_case (GBRunEvalContext *ec, GBValue *val, GBSelectCase *c,
 }
 
 gboolean
-gbrun_stmt_for (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
+gbrun_stmt_for (GBRunEvalContext  *ec,
+		const GBStatement *stmt,
+		gboolean           init)
 {
 	GBObjRef i;
 	GBValue  *to;
@@ -489,7 +550,9 @@ gbrun_stmt_for (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
 
 
 static gboolean
-gbrun_stmt_while (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
+gbrun_stmt_while (GBRunEvalContext  *ec,
+		  const GBStatement *stmt,
+		  gboolean           init)
 {
 	GBValue      *val;
 	GBBoolean     going = TRUE;
@@ -509,6 +572,224 @@ gbrun_stmt_while (GBRunEvalContext *ec, const GBStatement *stmt, gboolean init)
 	return TRUE;
 }
 
+gboolean
+gbrun_stmt_foreach (GBRunEvalContext  *ec,
+		    const GBStatement *stmt,
+		    gboolean           init)
+{
+	GBValue  *collection;
+	GBValue  *index;
+	GBValue **sval;
+	GBValue  *value;
+	char     *collection_name;
+	char     *index_name;
+	gboolean  is_array;
+	gboolean  is_collection;
+	gboolean  retval = FALSE;
+	GBBoolean ok;
+
+	collection_name = g_strconcat (
+		stmt->parm.foreach.var, "_collection", NULL);
+
+	index_name = g_strconcat (
+		stmt->parm.foreach.var, "_index", NULL);
+
+	if (init) { /* Setup */
+		GBValue *col;
+
+		col = gb_eval_context_eval (
+			GB_EVAL_CONTEXT (ec), 
+			stmt->parm.foreach.collection);
+
+		if (!col)
+			goto ret_free_names;
+
+		/* We need to put some stuff on the stack */
+		gbrun_stack_set  (ec, collection_name, col);
+		gb_value_destroy (col);
+
+		index = gb_value_new_long (0);
+	} else {
+		sval = gbrun_stack_get (ec, index_name);
+		if (!sval || !*sval)
+			goto ret_free_names;
+		index = gb_value_copy (GB_EVAL_CONTEXT (ec), *sval);
+	}
+
+	sval = gbrun_stack_get (ec, collection_name);
+	if (!sval || !*sval)
+		goto ret_free_names;
+	collection = *sval;
+
+	is_collection = gtk_type_is_a (collection->gtk_type, GBRUN_TYPE_COLLECTION);
+	is_array      = gtk_type_is_a (collection->gtk_type, GBRUN_TYPE_ARRAY);
+	ok            = FALSE;
+	value         = NULL;
+
+	if (!(is_collection || is_array)) {
+		gbrun_exception_firev (ec, _("Can only use For Each on a collection or array"));
+		goto ret_free_names;
+
+	} else if (is_collection) {
+		g_warning ("Cannot For Each over a collection yet");
+		goto ret_free_names;
+
+	} else if (is_array) {
+		GBRunArray *array = GBRUN_ARRAY (collection->v.obj);
+		GBValue    *ubound;
+
+		if (!gbrun_array_initialized (array)) {
+			gbrun_exception_firev (ec, _("For loop not initialized"));
+			goto ret_free_names;
+		}
+
+		if (gbrun_array_dimensions (array) > 1) {
+			gbrun_exception_firev (ec, _("Too many (%d) dimensions in "
+						     "foreach array"),
+					       gbrun_array_dimensions (array));
+			goto ret_free_names;
+		}
+
+		if (init) {
+			gb_value_destroy (index);
+			if (!(index = gbrun_array_lbound (ec, array, 1)))
+				goto ret_free_names;
+		}
+
+		if (!(ubound = gbrun_array_ubound (ec, array, 1)))
+			goto ret_free_names;
+
+		if (!gb_eval_compare (GB_EVAL_CONTEXT (ec), index,
+				      GB_EXPR_LE, ubound, &ok))
+			goto ret_free_names;
+
+		if (ok) {
+			GBObjRef      r;
+			const GBExpr *expr;
+
+			r.method = FALSE;
+			r.name   = NULL;
+			expr     = gb_expr_new_value (gb_value_copy (
+				GB_EVAL_CONTEXT (ec), index));
+			r.parms  = g_slist_prepend (NULL, (gpointer) expr);
+
+			value = gb_object_deref (
+				GB_EVAL_CONTEXT (ec),
+				GB_OBJECT (array),
+				&r, FALSE);
+
+			g_slist_free (r.parms);
+			gb_expr_destroy (expr);
+		}
+	}
+
+	if (value) {
+		gbrun_stack_set (ec, stmt->parm.foreach.var, value);
+		gb_value_destroy (value);
+	}
+
+	retval = TRUE;
+
+	if (ok)
+		gbrun_frame_stmts_push (ec, stmt->parm.foreach.body);
+	
+	{ /* Advance index */
+		GBValue *step;
+		GBValue *tmp;
+
+		step = gb_value_new_int (1);
+		tmp  = gb_eval_add (index, step);
+
+		gbrun_stack_set (ec, index_name, tmp);
+		
+		gb_value_destroy (tmp);
+		gb_value_destroy (step);
+	}	
+
+ ret_free_names:
+	g_free (index_name);
+	g_free (collection_name);
+
+	return retval;
+}
+
+/*
+ * stack_seek_root:
+ * 
+ *  Finds a stack element that was a function or sub or
+ * property call, instead of a construct like 'for'
+ **/
+static void
+stack_seek_root (GBRunEvalContext  *ec,
+		 GBRunSubFrame     *sf,
+		 GBRunSubFrame    **dest_sf,
+		 GSList           **dest_stmt)
+{
+	if (sf->eval_call) {
+		*dest_sf   = sf;   /* this frame */
+		*dest_stmt = NULL; /* finish it */
+	} else
+		stack_seek_root (ec, sf->parent, dest_sf, dest_stmt);
+}
+
+static void
+stack_find (GBRunEvalContext  *ec,
+	    GBRunSubFrame     *sf,
+	    GBStatementType    t,
+	    GBRunSubFrame    **dest_sf,
+	    GSList           **dest_stmt)
+{
+	if (sf->stmts) {
+		GBStatement *stmt = sf->stmts->data;
+		if (stmt->type == t) {
+			*dest_sf   = sf;
+			*dest_stmt = sf->stmts;
+			return;
+		}
+	}
+
+	stack_find (ec, sf->parent, t, dest_sf, dest_stmt);
+}
+
+static gboolean
+gbrun_stmt_exit (GBRunEvalContext  *ec,
+		 const GBStatement *stmt)
+{
+	GBRunFrame    *rf = gbrun_stack_frame (ec->stack);
+	GBRunSubFrame *dest_sf = NULL;
+	GSList        *dest_stmt = NULL;
+
+	switch (stmt->parm.exit) {
+	case GBS_EXIT_FUNCTION:
+	case GBS_EXIT_SUB:
+	case GBS_EXIT_PROPERTY:
+		stack_seek_root (ec, rf->cur, &dest_sf, &dest_stmt);
+		break;
+	case GBS_EXIT_DO:
+		stack_find (ec, rf->cur, GBS_DO, &dest_sf, &dest_stmt);
+		break;
+	case GBS_EXIT_FOR:
+		stack_find (ec, rf->cur, GBS_FOR, &dest_sf, &dest_stmt);
+		if (!dest_sf)
+			stack_find (ec, rf->cur, GBS_FOREACH, &dest_sf, &dest_stmt);
+		break;
+	default:
+		g_warning ("Unhandled exit stmt type");
+		break;
+	}
+
+	if (!dest_sf) {
+		gbrun_exception_firev (ec, _("Nothing to Exit"));
+		return FALSE;
+
+	} else {
+		gbrun_frame_crop_to_ptr (ec, dest_sf);
+		dest_sf->stmts = dest_stmt;
+
+		return TRUE;
+	}
+}
+
 /* 
  * Returns error flag.
  */
@@ -529,6 +810,7 @@ gbrun_stmt_evaluate (GBRunEvalContext *ec, const GBStatement *stmt, gboolean ini
 	case GBS_FOR:
 		return gbrun_stmt_for (ec, stmt, init);
 
+	case GBS_DO:
 	case GBS_WHILE:
 		return gbrun_stmt_while (ec, stmt, init);
 
@@ -562,7 +844,7 @@ gbrun_stmt_evaluate (GBRunEvalContext *ec, const GBStatement *stmt, gboolean ini
 				return FALSE;
 
 			for (l = stmt->parm.select.cases; l; l = l->next) {
-				if (handle_stmt_case (ec, val, l->data, &err))
+				if (gbrun_stmt_case (ec, val, l->data, &err))
 					break;
 			}
 		
@@ -644,7 +926,7 @@ gbrun_stmt_evaluate (GBRunEvalContext *ec, const GBStatement *stmt, gboolean ini
 		break;
 		
 	case GBS_GOTO:
-		return handle_stmt_goto (ec, stmt->parm.label);
+		return gbrun_stmt_goto (ec, stmt->parm.label);
 
 	case GBS_LABEL:
 		return TRUE;
@@ -661,12 +943,20 @@ gbrun_stmt_evaluate (GBRunEvalContext *ec, const GBStatement *stmt, gboolean ini
 	case GBS_SET:
 		return gbrun_stmt_set (ec, stmt); 
 
-	case GBS_WITH:
+	case GBS_REDIM:
+		return gbrun_stmt_redim (ec, stmt);
+
+	case GBS_EXIT:
+		return gbrun_stmt_exit (ec, stmt);
+
 	case GBS_FOREACH:
-	case GBS_DO:
+		return gbrun_stmt_foreach (ec, stmt, init);
+
+	case GBS_WITH:
 
 	default:
-		gbrun_exception_fire (ec, _("Unhandled statement"));
+		gbrun_exception_firev (ec, _("Unhandled statement '%s'"),
+				       gb_stmt_type (stmt));
 		return FALSE;
 	}
 
@@ -705,7 +995,7 @@ gbrun_stmts_evaluate (GBRunEvalContext *ec, GSList *stmts)
 				if (!strcmp (ec->on_error.label, "0"))
 					break;
 
-				if (handle_stmt_goto (ec, ec->on_error.label))
+				if (gbrun_stmt_goto (ec, ec->on_error.label))
 					goto stmt_eval_recheck;
 				else
 					break;
@@ -717,7 +1007,7 @@ gbrun_stmts_evaluate (GBRunEvalContext *ec, GSList *stmts)
 	}
 
 	if (stmt)
-		gbrun_frame_crop_to (ec, 0);
+		gbrun_frame_crop_to_depth (ec, 0);
 	
 	return stmt == NULL;
 }
