@@ -145,7 +145,7 @@ eval_to_penultimate (GBRunEvalContext *ec,
 		}
 
 		object = nxt->v.obj;
-		gb_object_ref (GB_EVAL_CONTEXT (ec), object);
+		gb_object_ref (object);
 		gb_value_destroy (nxt);
 	}
 
@@ -189,58 +189,17 @@ gbrun_eval_assign (GBRunEvalContext *ec,
 		
 	}
 	g_slist_free (try);
-
-	object = GB_OBJECT (ec->stack);
-
-	return gb_object_assign (GB_EVAL_CONTEXT (ec),
-				 GB_OBJECT (object),
-				 ref, value, FALSE);
-	/* FIXME: With ? */
-/*	if (!objref->next) { * Plain stack variable *
-		ref = objref->data;
-
-		if (ref->parms || ref->method) {
-			GBValue **t;
-
-			t = gbrun_stack_get (ec, ref->name);
-			if (t && (*t)->type == GB_VALUE_OBJECT &&
-			    (*t)->v.obj && GBRUN_IS_ARRAY ((*t)->v.obj))
-				return gbrun_array_assign (ec, (*t)->v.obj, ref, value);
-
-			gbrun_exception_firev (ec, "Can't assign to %s", ref->name);
-			return FALSE;
-		}
-
-		gbrun_stack_set (ec, ref->name, value);
-		return TRUE;
-	}
-
-	object = eval_to_penultimate (ec, objref);
-	if (!object) {
-		g_warning ("Nothing penultimate: wierd");
+	if (ec->options.explicit == TRUE) {
+		gbrun_exception_firev (
+			ec, _("variable '%s' not defined."), ref->name);
 		return FALSE;
-	}
-	
-	ref = g_slist_last ((GSList *) objref)->data;
-	if (ref->method) {
-		gbrun_exception_firev (ec, "Methods can't be lvalues");
-		return FALSE;
-	}
-
-	if (ref->parms) {
-		if (GBRUN_IS_ARRAY (object))
-			return gbrun_array_assign (ec, object, ref, value);
-		else {
-			gbrun_exception_firev (ec, "Unimplemented lvalue object dereference");
-			return FALSE;
-		}
 	} else {
-		gbrun_object_set_arg (ec, object, ref->name, value);
-		return TRUE;
-	}
+		object = GB_OBJECT (ec->stack);
 
-	gbrun_exception_firev (ec, "Unimplemented lvalue object dereference");
-	return FALSE;*/
+		return gb_object_assign (GB_EVAL_CONTEXT (ec),
+					 GB_OBJECT (object),
+					 ref, value, FALSE);
+	} 
 }
 
 GBValue *
@@ -286,7 +245,7 @@ gbrun_value_default_from_var (GBRunEvalContext *ec,
 				gb_eval_exception_firev (GB_EVAL_CONTEXT (ec),
 							 _("Unknown type '%s'"), var->type);
 			else
-				val = gb_value_new_default (GB_EVAL_CONTEXT (ec), t);
+				val = gb_value_new_default ((GBEvalContext *) ec, t);
 		}
 	}
 
