@@ -91,7 +91,7 @@ gbrun_func_instr (GBRunEvalContext *ec,
 		break;
 	case 2: /* Access, from Option Compare */
 	default:
-		gbrun_exception_fire(ec, "instr: unsupported comparison");
+		gbrun_exception_fire(ec, _("instr: unsupported comparison"));
 	}
 	if (start == 0 || start > 1 + hayst->len - needle->len)
 		return gb_value_new_int(0);
@@ -124,7 +124,7 @@ gbrun_func_instrb (GBRunEvalContext *ec,
 		break;
 	case 2: /* Access, from Option Compare */
 	default:
-		gbrun_exception_fire(ec, "instr: unsupported comparison");
+		gbrun_exception_fire(ec, _("instr: unsupported comparison"));
 	}
 	if (start == 0 || start > 1 + hayst->len - needle->len)
 		return gb_value_new_int(0);
@@ -162,7 +162,7 @@ gbrun_func_left (GBRunEvalContext *ec,
 	str = args [0]->v.s;
 
 	if (length > str->len)
-		return gbrun_exception_fire (ec, "length larger than string");
+		return gbrun_exception_fire (ec, _("length larger than string"));
 	
 	str->str[length]='\0';
 	ans = gb_value_new_string_chars (str->str);
@@ -183,7 +183,7 @@ gbrun_func_leftb (GBRunEvalContext *ec,
 	str = args [0]->v.s;
 
 	if (length > str->len)
-		return gbrun_exception_fire (ec, "length larger than string");
+		return gbrun_exception_fire (ec, _("length larger than string"));
 	
 	ans = gb_value_new_int (sizeof (*str->str) * length);
 
@@ -254,10 +254,10 @@ gbrun_func_mid (GBRunEvalContext *ec,
 	last = start + length;
 		
 	if (length > str->len)
-		return gbrun_exception_fire (ec, "offset larger than string");
+		return gbrun_exception_fire (ec, _("offset larger than string"));
 
 	if (last > str->len)
-		return gbrun_exception_fire (ec, "last beyond end of string");
+		return gbrun_exception_fire (ec, _("last beyond end of string"));
 
 	str->str [last]='\0';
 	ans = gb_value_new_string_chars (str->str + start);
@@ -283,10 +283,10 @@ gbrun_func_midb (GBRunEvalContext *ec,
 	last = start + length;
 		
 	if (length > str->len)
-		return gbrun_exception_fire (ec, "offset larger than string");
+		return gbrun_exception_fire (ec, _("offset larger than string"));
 
 	if (last > str->len)
-		return gbrun_exception_fire (ec, "last beyond end of string");
+		return gbrun_exception_fire (ec, _("last beyond end of string"));
 
 	ans = gb_value_new_int (sizeof (*str->str) * length);
 
@@ -310,7 +310,7 @@ gbrun_func_right (GBRunEvalContext *ec,
 	first = str->len - length;
 
 	if (length > str->len)
-		return gbrun_exception_fire(ec, "offset beyond end of string");
+		return gbrun_exception_fire(ec, _("offset beyond end of string"));
 		
 	ans = gb_value_new_string_chars (str->str + first);
 	
@@ -333,7 +333,7 @@ gbrun_func_rightb (GBRunEvalContext *ec,
 	first = str->len - length;
 
 	if (length > str->len)
-		return gbrun_exception_fire(ec, "offset beyond end of string");
+		return gbrun_exception_fire(ec, _("offset beyond end of string"));
 		
 	ans = gb_value_new_int (sizeof (*str->str) * length);
 	
@@ -549,7 +549,7 @@ gba_string_register (GBEvalContext *ec)
 
 	gba_object = gtk_type_new (
 		gbrun_object_subclass_simple (GBRUN_TYPE_OBJECT, ITEM_NAME));
-	gbrun_global_add (GB_OBJECT (gba_object));
+	gbrun_global_add (GB_OBJECT (gba_object), "string");
 
 	gba = GBRUN_OBJECT_GET_CLASS (gba_object);
 
