@@ -58,6 +58,18 @@ cmdbutton_mouseup (GtkWidget *button, GBRunEvalContext *ec)
 	gbrun_form_item_invoke (ec, button, "_MouseUp");
 }
 
+static void
+cmdbutton_mouseover (GtkWidget *button, GBRunEvalContext *ec)
+{
+	gbrun_form_item_invoke (ec, button, "_MouseOver");
+}
+
+static void
+cmdbutton_mouseout (GtkWidget *button, GBRunEvalContext *ec)
+{
+	gbrun_form_item_invoke (ec, button, "_MouseOut");
+}
+
 /**
  * cmdbutton_setarg:
  *   @ec
@@ -105,7 +117,6 @@ cmdbutton_setarg (GBRunEvalContext *ec,
 	case DEFAULT:
 		/* FIXME: Is this the right function ? */
 		gtk_widget_set (GTK_WIDGET (bwidget), "can_default", val->v.bool, NULL);
-		g_warning ("FIXME: hacked");
 /*		if (val->v.bool)
 		gtk_window_set_default (button->item.form->window, button->item.widget);*/
 		return TRUE;
@@ -185,6 +196,10 @@ gbrun_cmdbutton_construct (GBRunEvalContext *ec, GBRunFormItem *item)
 				(GtkSignalFunc) cmdbutton_mousedown, ec);
 	gtk_signal_connect (GTK_OBJECT (w), "released",
 				(GtkSignalFunc) cmdbutton_mouseup, ec);
+	gtk_signal_connect (GTK_OBJECT (w), "enter",
+				(GtkSignalFunc) cmdbutton_mouseover, ec);
+	gtk_signal_connect (GTK_OBJECT (w), "leave",
+				(GtkSignalFunc) cmdbutton_mouseout, ec);
 }
 
 static void
