@@ -480,7 +480,7 @@ gbrun_form_init (GBRunEvalContext  *ec,
 		gbrun_form_pass_properties (ec, GBRUN_OBJECT (item), i);
 		
 		/* Add it to the stack */
-		gbrun_object_ref (ec, item);
+		gbrun_object_ref (item);
 		gbrun_stack_add  (ec, i->name,
 				  gb_value_new_object (GB_OBJECT (item)),
 				  GBRUN_STACK_MODULE);
