@@ -27,12 +27,14 @@ gbrun_objects_register (GBEvalContext *ec)
 	gbrun_timer_register ();
 	gbrun_picturebox_register ();
 	gbrun_listbox_register ();
-	gbrun_frame_register();
+	gbrun_frame_register ();
+	gbrun_screen_register ();
 }
 
 void
 gbrun_objects_shutdown (void)
 {
+	gbrun_screen_shutdown ();
 	gbrun_frame_shutdown ();
 	gbrun_listbox_shutdown ();
 	gbrun_checkbox_shutdown ();
