@@ -2,8 +2,8 @@
  * Gnome Basic controller.
  *
  * Authors:
- *     Jody Goldberg (jgoldberg@home.com)
  *     Michael Meeks (mmeeks@gnu.org)
+ *     Jody Goldberg (jgoldberg@home.com)
  *
  * Copyright 2000, Helix Code, Inc.
  */
@@ -38,7 +38,7 @@ const struct poptOption gb_popts [] = {
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
-GBLexerStream *
+static GBLexerStream *
 file_to_stream (const char *filename)
 {
 	guint8            *data;
@@ -81,7 +81,8 @@ file_to_stream (const char *filename)
 	return ans;
 }
 
-const GBParseData *
+#if 0
+static const GBParseData *
 parse_file (const char *filename)
 {
 	GBLexerStream     *ls;
@@ -103,8 +104,9 @@ parse_file (const char *filename)
 
 	return module;
 }
+#endif
 
-GBLexerStream *
+static GBLexerStream *
 stream_provider (GBRunEvalContext *ec,
 		 const char       *name,
 		 gpointer          user_data)
@@ -431,6 +433,7 @@ main (int argc, char * argv[])
 		const char test [] = 
 			"print \"*Testing string compilation*\"\n"
 			"Dim j As Long\n"
+			"Dim i As Long\n" 
 			"j = 1\n"
 			"For i = 1 To 6\n"
 			"	j = j * i\n"
